unit adcontrl;

{ Various handle-related operations.
  Change initad, startad, readad for particular A-to-D board }

interface
uses dos,crt,graph,mycrt,adIBM,adRTD,ad2814,adJOY,adMOUSE;

type scalelist = array[0..3,0..1] of integer;
     configtype = record
                   adkind: integer;
                   adport: integer;
                   boardnum: integer;
                   pole: integer;
                   scale: scalelist;
                   chan: array[0..3] of integer;
                   speed: char;
                  end;

var  adconfig: configtype;
     processor: char;
     usekbd: boolean;
     keypicked: boolean;

{$F+}
     mousex,mousey: ^integer;
{$F-}
function SetATOD: boolean;    { call at start of program }
procedure adstart(channel: integer);
function adread(channel: integer): integer;

implementation


{ -------------------------------------------------------------------------}


const maxad = 5;
      adnames: array[0..5] of string[40] =
               ('IBM DAA','Real Time Devices Model 200',
                'Data Translation DT2814','Game Joystick','Mouse','Keyboard');

var  configfile: file of configtype;
     ch: char;
     Adnumber,result: integer;

function gethexbyte: integer;
var n,i: integer;
    s: string[5];
begin
 readln(s);
 n := 0;
 for i := 1 to length(s) do
 begin
  n := n * 16;
  if s[i] in ['0'..'9'] then n := n + ord(s[i]) - ord('0')
  else
  if s[i] in ['A'..'F'] then n := n + ord(s[i]) - ord('0') - 7
  else
   begin
    gethexbyte := maxint;
    exit;
   end;
 end;
 gethexbyte := n;
end;


function adinit: boolean;
begin
 with adconfig do
 case adkind of
  0: adinit :=
     initIBMAD(adport,boardnum,pole,scale);
  1: adinit :=
     initRTD(adport,boardnum,pole,scale);
  2: adinit :=
     init2814(adport,boardnum,pole,scale);
  3: adinit :=
     initJOY(adport,boardnum,pole,scale);
  4:
     begin
      adinit :=
       initMOUSE(adport,boardnum,pole,scale);
      mousex := @ADmouse.mousex;
      mousey := @ADmouse.mousey;
     end;
 end;
end;

procedure adstart(channel: integer);
var chanl: integer;
begin
 chanl := adconfig.chan[channel];
 with adconfig do
 case adkind of
  0: startIBMAD(chanl);
  1: startRTD(chanl);
  2: start2814(chanl);
  3: startJOY(chanl);
  4: startMOUSE(chanl);
 end;
end;

function adread(channel: integer): integer;
var chanl: integer;
begin
 chanl := adconfig.chan[channel];
 with adconfig do
 case adkind of
  0: adread := readIBMAD(chanl);
  1: adread := readRTD(chanl);
  2: adread := read2814(chanl);
  3: adread := readJOY(chanl);
  4: adread := readMOUSE(chanl);
 end;
end;

function getmenu: integer;
var i,n: integer;
begin
 window(1,2,80,25);
 repeat
  clrscr;
  for i := 0 to maxad - 1 do
   begin
    gotoxy(1,2+2*i); write(i+1:2,'  ',adnames[i]:35);
   end;
  if usekbd then
   begin
    gotoxy(1,4+2*maxad); write(maxad+1:2,'  ','KEYBOARD':35);
   end;
  gotoxy(1,20); Write('Enter choice number: ');
  readln(n);
  n := n - 1;
 until (
         (usekbd and (n < 6)) or
         (n < 5)
       );
 getmenu := n;
 window(1,1,80,25);
end;

function SetATOD: boolean;
var ADOK: boolean;
    i: integer;
begin
 assign(configfile,'ADconfig');
 {$i-}
 reset(configfile);
 {$i+}
 result := IOResult;
 if Result = 0 then
  begin
   {$i-}
   read(configfile,adconfig);
   {i+}
   result := IOResult;
   if result = 0 then
   begin
    close(configfile);
    processor := adconfig.speed;
   end
  end
 else
  with adconfig do
  begin
   repeat
    gotoxy(12,10); write('Slow (PC) or Fast (AT) computer? S/F: '); clreol;
    processor := readkey; writeln(processor);
   until processor in ['s','S','f','F'];
   adconfig.speed := processor;
   adkind := getmenu;
   for i := 0 to 3 do adconfig.chan[i] := i;
   if adkind < 4 then
   begin
    gotoxy(12,14);
    write(
    'Enter actual channel numbers (0..15) to be used for channels 0,1,2,3:');
    repeat
     gotoxy(12,15); clreol;
     write('Channel 0: '); readln(adconfig.chan[0]);
    until adconfig.chan[0] in [0..15];
    repeat
     gotoxy(12,16); clreol;
     write('Channel 1: '); readln(adconfig.chan[1]);
    until adconfig.chan[1] in [0..15];
    repeat
     gotoxy(12,17); clreol;
     write('Channel 2: '); readln(adconfig.chan[2]);
    until adconfig.chan[2] in [0..15];
    repeat
     gotoxy(12,18); clreol;
     write('Channel 3: '); readln(adconfig.chan[3]);
    until adconfig.chan[3] in [0..15];
   end;
   clrscr;
   gotoxy(12,8); writeln(adnames[adkind],'  CHOSEN');
   delay(1000);
   case adkind of
    0: begin     { IBM DAA }
        repeat
         gotoxy(12,12); write('Number of board, 0 to 3: '); clreol;
         readln(boardnum);
        until boardnum in [0..3];
        adport := $240; fillchar(scale,sizeof(scale),0);
       end;
    1: begin    { Real Time Devices }
        repeat
         gotoxy(12,12);
         write('Hexadecimal port address of board (normal 240): '); clreol;
         adport := gethexbyte;
        until adport < $1000;
        pole := 1; boardnum := 0; fillchar(scale,sizeof(scale),0);
       end;
    2: begin    {Data Translation DT2814}
        repeat
         gotoxy(12,12);
         write('Hexadecimal port address of board (normal 220): '); clreol;
         adport := gethexbyte;
        until adport < $1000;
        repeat
         gotoxy(12,14);
         write('Unipolar (0..5V) or Bipolar (-5V to 5V): U/B: '); clreol;
         ch := readkey
        until ch in ['u','U','b','B'];
        if ch in ['u','U'] then pole := 1 else pole := 2;
        boardnum := 0;
        fillchar(scale,sizeof(scale),0);
       end;
    3: begin   {JOYSTICK}
        adport := $200;
        boardnum := 0;
        pole := 1;
        fillchar(scale,sizeof(scale),0); {trigger calibration}
       end;
    4: begin   {MOUSE}
        adport := 0;
        boardnum := 0;
        pole := 1;
        repeat
         gotoxy(12,12);
         write('Mouse sensitivity, counts/inch, 50..1000: '); clreol;
         readln(scale[0,0]);
        until (scale[0,0] >= 50) and (scale[0,0] <= 1000);
       end;
   end;
  end;
  keypicked := (adconfig.adkind = 5);
  if keypicked and not usekbd then
   begin
    erase(configfile);
    writeln;
    writeln(
    'Keyboard not allowed. Configuration file erased, please restart program');
    writeln('and choose an an analogue input device from menu');
    halt;
   end;
  if not keypicked then ADOK := ADinit  { initialize }
  else ADOK := true;
  if adok then
  begin
   rewrite(configfile);
   write(configfile,adconfig);
   close(configfile);
  end;
  SetATOD := ADOK;   { report success }
end;

end.  { of unit }

